#if ardent || ultrix || mips || _AIX370 || __mips
/***********************************************************************
These functions provide a Fortran caller with BSD UNIX-like bit primitives
on the Ardent Titan.

Here is the Sun OS "man 3f bit" output:


NAME
     bit - and, or, xor, not, rshift, lshift, bic, bis, bit, set-
     bit functions

SYNOPSIS
     (generic) function and (word1, word2)

     (generic) function or (word1, word2)

     (generic) function xor (word1, word2)

     (generic) function not (word)

     (generic) function rshift (word, nbits)

     (generic) function lshift (word, nbits)

     subroutine bic (bitnum, word)
     integer*4 bitnum, word

     subroutine bis (bitnum, word)
     integer*4 bitnum, word

     subroutine setbit (bitnum, word, state)
     integer*4 bitnum, word, state

     logical function bit (bitnum, word)
     integer*4 bitnum, word

DESCRIPTION
     The and, or, xor, not, rshift, and lshift functions are gen-
     eric  functions  expanded  inline  by   the compiler.  Their
     arguments must be integer or logical values (short or long).
     The returned value has the data type of the first argument.

     Bits are numbered such that bit 0 is the  least  significant
     bit, and bit 31 is the most significant.

     and  computes the bitwise `and' of its arguments.

     or   computes the bitwise `or' of its arguments.

     xor  computes the bitwise `exclusive or' of its arguments.

     not  returns the bitwise complement of its argument.

     lshift
          is a logical left shift with no end around carry.

     rshift
          is an arithmatic right shift with sign  extension.   No
          test is made for a reasonable value of nbits.

     Bic, bis, and setbit are external subroutines which  operate
     on integer*4 arguments.

     bis  sets bitnum in word.

     bic  clears bitnum in word.

     setbit
          sets bitnum in word to 1 if state is nonzero and clears
          it otherwise.

     bit  is an external function which tests bitnum in word  and
          returns  .true.  if  bitnum  is  a 1 (one), and returns
          .false. if bitnum is a 0 (zero).

FILES
     /usr/lib/libF77.a

[16-Nov-89]
***********************************************************************/

#if ultrix || (mips && !ardent) || (__mips && !ardent)
#define BIC bic_
#define BIS bis_
#define BIT bit_
#endif /* ultrix || (mips && !ardent) || (__mips && !ardent) */

#if _AIX370
#define AND and_
#define BIC bic_
#define BIS bis_
#define BIT bit_
#define LSHIFT lshift_
#define NOT not_
#define OR or_
#define RSHIFT rshift_
#define SETBIT setbit_
#define XOR xor_
#endif

int
AND(int *a,int *b)
{
	return (*a & *b);
}

void
BIC(int *bitnum, int *word)
{
	*word &= ~(1 << *bitnum);
}

void
BIS(int *bitnum, int *word)
{
	*word |= (1 << *bitnum);
}

int
BIT(int *bitnum, int *word)
{
	return ((*word & (1 << *bitnum)) ? 1 : 0);
}

int
LSHIFT(int *word,int *nbits)
{
	return ((*word) << (*nbits));
}

int
NOT(int *a)
{
	return (~(*a));
}

int
OR(int *a,int *b)
{
	return (*a | *b);
}

int
RSHIFT(int *word,int *nbits)
{
	return ((*word) >> (*nbits));
}

void
SETBIT(int *bitnum, int *word,int *state)
{
	if (*state)
		*word |= (1 << *bitnum);
	else
		*word &= ~(1 << *bitnum);
}

int
XOR(int *a,int *b)
{
	return (*a ^ *b);
}
#endif /* ardent || ultrix || mips  || _AIX370 || __mips */
